// Step 1: Clean raw data files to keep only the variables of interest. This step helps reduce the size of DTA files, making them more manageable. 
//		   NOTE: this example file cleans 5 of the processed files from s0_zip_txt_dta: All address, Contact Info, Identifiers, Industry classifications, and Legal info.
//			     a similar procedure can be implemented for processing any of the other files.

* 0) Preliminaries
*** Cleaning
	clear
	cls
	set more off
	set rmsg on																	// Keeps track of length of process
	
*** Install codes
   ssc install fs
   ssc install confirmdir
   ssc install chunky
   
*** Directories
global unzip "C:\Program Files\7-Zip\7z"				      					// Location of 7z executable program
global loc "CHANGE TO RELEVANT DRIVE"											// CHANGE DRIVE WHERE FORLDERS ARE LOCATED
global raw "$loc\firm_description\zip" 											// Location of zip files (SUGGESTION: COPY INTO THIS FOLDER ONLY THE ZIP FILES THAT YOU WANT TO CONVERT. THE GUIDE PROCESSES 12 OF THE 32 LOCATED ON THE ORBIS DRIVE IN FOLDER ORBIS DEC TEXT)
global txt "$loc\firm_description\txt"											// Output text files
global dta "$loc\firm_description\dta"											// Output data files 

global codes "$loc\processing_codes"											// Locataion to save log files
	
* 1) All addresses files
clear
fs "$dta\intermediate\*.dta"
	return list
set obs 1 
	gen fn = ""
	gen id = 0
foreach f in `r(files)' {
	insobs 1
	replace id = id[_n-1] + 1 if _n > 1
	local name `f'
	replace fn = "`name'" if _n == _N
}
drop if id == 0
keep if strpos(fn,"address") > 0
split fn, p(".") gen(fname)
	keep fname1 id
	rename fname1 fname
	gen string = substr(fname,-2,2)
	keep id string
		drop id
		gen id = _n
	 disp "List of files in processing loop"
save "$loc\firm_description\process.dta", replace
	qui sum id
	return list
forval i = `r(min)'(1)`r(max)' {
	use "$loc\firm_description\process.dta", clear
	noisily keep if id == `i'
	global st `=string[1]'	
	disp "$st"	
	use "$dta\intermediate\all addresses00$st.txt.dta", clear
	rename ïbvdidnumber bvdid
	gen ISO = substr(bvdid,1,2)
	
	drop streetnobuildingetcline1native streetnobuildingetcline2native streetnobuildingetcline3native ///
	     streetnobuildingetcline4native citynative country telephonenumber faxnumber	 
	
	foreach x in streetnobuildingetcline1 streetnobuildingetcline2 streetnobuildingetcline3 ///
	       streetnobuildingetcline4 postcode city countryisocode regionincountry typeofregionincountry addresstype {
			tostring `x', replace
			replace `x' = "" if `x' == "."
		 }

	egen miss = rowmiss(streetnobuildingetcline1 streetnobuildingetcline2 streetnobuildingetcline3 ///
	        streetnobuildingetcline4 postcode city countryisocode regionincountry)
		drop if miss == 8
		drop miss
	save "$dta\intermediate\all addresses00$st.txt.dta", replace
}

erase "$loc\firm_description\process.dta"

* 2) Contact info
clear
fs "$dta\intermediate\*.dta"
	return list
set obs 1 
	gen fn = ""
	gen id = 0
foreach f in `r(files)' {
	insobs 1
	replace id = id[_n-1] + 1 if _n > 1
	local name `f'
	replace fn = "`name'" if _n == _N
}
drop if id == 0
keep if strpos(fn,"contact") > 0
split fn, p(".") gen(fname)
	keep fname1 id
	rename fname1 fname
	gen string = substr(fname,-2,2)
	keep id string
		drop id
		gen id = _n
	 disp "List of files in processing loop"
save "$loc\firm_description\process.dta", replace
	qui sum id
	return list
forval i = `r(min)'(1)`r(max)' {
	use "$loc\firm_description\process.dta", clear
	noisily keep if id == `i'
	global st `=string[1]'	
	disp "$st"	
	use "$dta\intermediate\contact info00$st.txt.dta", clear
	rename ïbvdidnumber bvdid
	gen ISO = substr(bvdid,1,2)
	
	drop name_native streetnobuildingetcline1native streetnobuildingetcline2native streetnobuildingetcline3native ///
		 streetnobuildingetcline4native citynative telephonenumber faxnumber websiteaddress emailaddress nuts*	 
	
	foreach x in streetnobuildingetcline1 streetnobuildingetcline2 streetnobuildingetcline3 ///
			streetnobuildingetcline4 postcode city country countryisocode metropolitanareainus ///
			stateorprovinceinusorcanada countyinusorcanada regionincountry typeofregionincountry {
			tostring `x', replace
			replace `x' = "" if `x' == "."
		 }

	egen miss = rowmiss(streetnobuildingetcline1 streetnobuildingetcline2 streetnobuildingetcline3 ///
		streetnobuildingetcline4 postcode city countryisocode stateorprovinceinusorcanada)
		drop if miss == 8
		drop miss
	save "$dta\intermediate\contact info00$st.txt.dta", replace
}

erase "$loc\firm_description\process.dta"

* 3) Identifiers
clear
fs "$dta\intermediate\*.dta"
	return list
set obs 1 
	gen fn = ""
	gen id = 0
foreach f in `r(files)' {
	insobs 1
	replace id = id[_n-1] + 1 if _n > 1
	local name `f'
	replace fn = "`name'" if _n == _N
}
drop if id == 0
keep if strpos(fn,"identifiers") > 0
split fn, p(".") gen(fname)
	keep fname1 id
	rename fname1 fname
	gen string = substr(fname,-2,2)
	keep id string
		drop id
		gen id = _n
	 disp "List of files in processing loop"
save "$loc\firm_description\process.dta", replace
	qui sum id
	return list
forval i = `r(min)'(1)`r(max)' {
	use "$loc\firm_description\process.dta", clear
	noisily keep if id == `i'
	global st `=string[1]'	
	disp "$st"	
	use "$dta\intermediate\identifiers00$st.txt.dta", clear
	rename ïbvdidnumber bvdid
	gen ISO = substr(bvdid,1,2)
	
	drop ipidentificationnumber ipidentificationlabel othercompanyidnumber statisticalnumber v13 isinnumber
	
	foreach x in nationalidnumber nationalidlabel nationalidtype traderegisternumber ///
	        europeanvatnumber leilegalentityidentifier tickersymbol {
			tostring `x', replace 
			replace `x' = "" if `x' == "."
		 }

	foreach x in vattaxnumber {
		tostring `x', replace format("%22.0f")
		replace `x' = "" if `x' == "."
	}

	egen miss = rowmiss(nationalidnumber nationalidlabel nationalidtype traderegisternumber vattaxnumber ///
	        europeanvatnumber leilegalentityidentifier)
		drop if miss == 7
		drop miss
	save "$dta\intermediate\\identifiers00$st.txt.dta", replace
}

erase "$loc\firm_description\process.dta"

* 4) Industry classifications
clear
fs "$dta\intermediate\*.dta"
	return list
set obs 1 
	gen fn = ""
	gen id = 0
foreach f in `r(files)' {
	insobs 1
	replace id = id[_n-1] + 1 if _n > 1
	local name `f'
	replace fn = "`name'" if _n == _N
}
drop if id == 0
keep if strpos(fn,"industry") > 0
split fn, p(".") gen(fname)
	keep fname1 id
	rename fname1 fname
	gen string = substr(fname,-2,2)
	keep id string
		drop id
		gen id = _n
	 disp "List of files in processing loop"
save "$loc\firm_description\process.dta", replace
	qui sum id
	return list
forval i = `r(min)'(1)`r(max)' {
	use "$loc\firm_description\process.dta", clear
	noisily keep if id == `i'
	global st `=string[1]'	
	disp "$st"	
	use "$dta\intermediate\industry classifications00$st.txt.dta", clear
	rename ïbvdidnumber bvdid
	gen ISO = substr(bvdid,1,2)
	
	cap {
	noisily drop nacerev2mainsection nationalindustryclassificationus primarycodesinthisclassification primarycodeinnationalindustrycla secondarycodesinthisclassificati ///
	     secondarycodeinnationalindustryc nacerev2corecodetextdescription nacerev2primarycodetextdescripti nacerev2secondarycodetextdescrip ///
		 naics2012corecodetextdescription naics2012primarycodetextdescript naics2012secondarycodetextdescri ussiccorecodetextdescription ///
		 ussicprimarycodetextdescription ussicsecondarycodetextdescriptio bvdmajorsector
	}
	cap {
		noisily replace ussicprimarycodes = "" if ussicprimarycodes == "NULL"
		noisily replace ussicprimarycodes = subinstr(ussicprimarycodes," ","",.)
			noisil destring ussicprimarycodes, replace
		}
	cap {
		noisily replace ussicsecondarycodes = "" if ussicsecondarycodes == "NULL"
		noisily replace ussicsecondarycodes = "" if ussicsecondarycodes == "JHB"
		noisily replace ussicsecondarycodes = subinstr(ussicsecondarycodes," ","",.)
			noisil destring ussicsecondarycodes, replace
		}		
	cap {
		noisily replace ussiccorecode3digits = "" if ussiccorecode3digits == "NUL"
		noisily replace ussiccorecode3digits = subinstr(ussiccorecode3digits," ","",.)
			noisily destring ussiccorecode3digits, replace
		}
	cap {
		noisily destring nacerev2secondarycodes, replace
	}
		 
	egen miss = rowmiss(nacerev2corecode4digits nacerev2primarycodes nacerev2secondarycodes naics2012corecode4digits naics2012primarycodes ///
	     naics2012secondarycodes ussiccorecode3digits ussicprimarycodes ussicsecondarycodes)
		drop if miss == 9
		drop miss
	save "$dta\intermediate\industry classifications00$st.txt.dta", replace
}

erase "$loc\firm_description\process.dta"

* 5) Legal info
clear
fs "$dta\intermediate\*.dta"
	return list
set obs 1 
	gen fn = ""
	gen id = 0
foreach f in `r(files)' {
	insobs 1
	replace id = id[_n-1] + 1 if _n > 1
	local name `f'
	replace fn = "`name'" if _n == _N
}
drop if id == 0
keep if strpos(fn,"legal") > 0
split fn, p(".") gen(fname)
	keep fname1 id
	rename fname1 fname
	gen string = substr(fname,-2,2)
	keep id string
		drop id
		gen id = _n
	 disp "List of files in processing loop"
save "$loc\firm_description\process.dta", replace
	qui sum id
	return list
forval i = `r(min)'(1)`r(max)' {
	use "$loc\firm_description\process.dta", clear
	noisily keep if id == `i'
	global st `=string[1]'	
	disp "$st"	
	use "$dta\intermediate\\legal info00$st.txt.dta", clear
	rename ïbvdidnumber bvdid
	gen ISO = substr(bvdid,1,2)
	
	drop alsoknownasname reasonoffilingexemption categoryofthecompany delistedcomment ///
	     norecentfinancialsflag historicalrecordflag historicalrecordsince informationprovider

	foreach x in previouscompanyname status standardisedlegalform ///
	     nationallegalform stateofincorporationinus typeofentity listeddelistedunlisted ///
		 mainexchange {
			tostring `x', replace
	}
		 
	egen miss = rowmiss(previouscompanyname namechangedate status statusdate standardisedlegalform ///
	     nationallegalform dateofincorporation stateofincorporationinus typeofentity listeddelistedunlisted ///
		 delisteddate mainexchange ipodate)
		drop if miss == 13
		drop miss
	save "$dta\intermediate\legal info00$st.txt.dta", replace
}

erase "$loc\firm_description\process.dta"


 